/** Eve Manager gateway class
 * 
 * @class Gateway
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function Gateway(){
	var self = this;
	
	/** Gateway Id
	 * @public
	 * @default undefined
	 */
	this.id = undefined;
	
	/** Gateway Unique Id
     * @private 
     * @type Integer
     * @default undefined
     */
    this.UId = undefined;
	
	/** Gateway name
	 * @private 
	 * @type string
	 * @default undefined
	 */
	var m_strName = undefined;
	
	/** Gateway protocol
	 * @private 
	 * @type string
	 * @default undefined
	 */
	var m_strProtocol = undefined;
	
	/** Gateway data (group data contains only group name)
	 * @private
	 * @type object
	 * @default new Object()
	 */
	var m_Data = new Object();
	
	//Methods
	
	/** Add new parameter and set value - Register action
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 * @public
	 */
	this.addParamVal = function(key,value){
		//Check if key already exist
		if(m_Data[key] != undefined){ return 1; }
		
		m_Data[key] = value;
		return 0;
	};
	
	/** Set parameter value
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setParamVal = function(key,value){
	    //console.log(m_Data);
		
		//Check if key already exist
		if(m_Data[key] == undefined){ return 1; }
		
		m_Data[key] = value;
		return 0;
	};
	
	/** Get parameter value
	 * @param {string} key Key of memory area
	 * @returns {Mixed data} Data by key
	 */
	this.getParamVal = function(key){
		return m_Data[key];
	};
	
	/** Remove parameter from data
     * @param {string} key Key of memory area
     * @returns {Integer} Error description (0 = Ok; otherwise = Error)
     */
    this.removeParam = function(key){
        delete m_Data[key];
    };
	
	/** Set gateway name (name parameter need to be unique)
	 * @param str name of gateway
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setName = function(strName){
		// Check input parameter is a string
		if(typeof strName != "string"){
			console.error("Type of input parameter is not a string");
			return 1;
		}
    	
    	//Set name
		m_strName = strName;
		return 0;
		
		//Check unique name
		/*if(self.m_oNode == undefined){
		    console.warn("Gateway reference node is undefined");
		}
		else{
		    var oParent = self.m_oNode.getParent();
		    var n = 1;
            var bFree = false;
		    
		    //console.log(m_strName);
		    //console.log(oParent);
		    
		    //Not on create gateway set name
		    if(oParent == undefined){ console.error("Parent node is undefined inside Gateway set name"); return; }
		    
		    //Find if name is unique 
            while(!bFree){
                var bIsEqual = false;
                
                //Parse all "brothers" name
                for(var i=0; i<oParent.children.length; ++i){
                    var oCompareGw = oParent.children[i].getData();
                    
                    //Find same name
                    if(oCompareGw !== self && oCompareGw.getName() == strName){
                        strName = self.getName() +" "+ n;
                        bIsEqual = true;
                        n++;
                        break;
                    }
                }
                
                if(!bIsEqual){
                    bFree = true;
                }
                
                //Extra end
                if(n > 1000){ console.warn("While cicle overflow!"); bFree = true; }
            }
            
		    m_strName = strName;                                                          //Set correct name
		}
		
		return 0;*/
	};
	
	/** Get gateway name
	 * @returns {string} Type of component
	 */
	this.getName = function(){
		return m_strName;
	};
	
	/** Set gateway protocol
	 * @param {string} strName name of gateway
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setProtocol = function(strProtocol){
		// Check input parameter is a string
		if(typeof strProtocol != "string"){
			console.error("Type of input parameter is not a string");
			return 1;
		}
		
		m_strProtocol = strProtocol;
		/*
		var iErr = self.setParamVal("protocol",strProtocol);                             //Set protocol also like a parameter (need ghost scenarios filter)
		if(iErr != 0){
		    self.addParamVal("protocol",strProtocol);
		}*/
		
		return 0;
	};
	
	/** Get gateway protocol
	 * @returns {string} Type of component
	 */
	this.getProtocol = function(){
		return m_strProtocol;
	};
	
	/** Get gateway data
     * @returns {string} Type of component
     */
    this.getData = function(){
        return m_Data;
    };
	
	/** Check is special */
	this.isSpecial = function(){
	    return (self.getParamVal("special") != undefined)?true:false;
	};
	
	/** Serialized */
	
	this.encodeJSON = function(){
	    
	    //Set instance variables
	    var oSerialized = {
	        UId: self.UId,
            name: m_strName,
            protocol: m_strProtocol
	    };
	    
	    //Set data parameters
	    for(var k in m_Data){
	        oSerialized[k] = m_Data[k];
	    }
	    
	    return oSerialized;
	};
	
	/** Set Serialized data for Eve db
	 * @param {string} iStr Serialized string set from component html
	 */
	this.setSerializedData = function(iStr){
	    self.m_Data["serialize"] = iStr;
	};
	
	/** Get Serialized data for Eve db
	 * @returns {string} Serialized string set from component html
	 */
	this.getSerializedData = function(){
	    //return self.m_Data["serialize"];
	    return m_Data;
	};
	
	/** Overwrite toString() method
     * @returns {string} String that describe Gateway object
     */
    this.toString = function(){
        return "[Gateway] - Unique Id: "+ self.UId + "; name: "+ m_strName + "; protocol: "+ m_strProtocol;
    };
}