﻿/**
 * @class Relation
 *
 * @description Eve Manager Relation Class
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 */
"use strict";

function Relation(){
	//Instance variables (private)
	var self = this;
	
	var init = function(){
    };
	
	/** Relation Id
	 * @public
	 * @default undefined
	 */
	this.id = undefined;

	/** Relation data (group data contains only group name)
	 * @public
	 * @type object
	 * @default new Object()
	 */
	this.m_Data = new Object();

    /**
     *
     */
    this.isComplete = false;
	
	//Methods
	
	/** Set parameter value
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 */
	this.setParamVal = function(key,value){
		self.m_Data[key] = value;

        if(self.m_Data["source"] != undefined && self.m_Data["destination"] != undefined && self.m_Data["operator"] != undefined && self.m_Data["checkTime"] != undefined){
            self.isComplete = true;
        }
        else{
            self.isComplete = false;
        }
	};
	
	/** Get parameter value
	 * @param {string} key Key of memory area
	 * @returns {mixed}
	 */
	this.getParamVal = function(key){
	    return self.m_Data[key];
	};
	
	/** Remove parameter from data
     * @param {string} key Key of memory area
     * @returns {Integer} Error description (0 = Ok; otherwise = Error)
     */
	this.removeParam = function(key){
	    delete self.m_Data[key];

        if(self.m_Data["source"] != undefined && self.m_Data["destination"] != undefined && self.m_Data["operator"] != undefined && self.m_Data["checkTime"] != undefined){
            self.isComplete = true;
        }
        else{
            self.isComplete = false;
        }
	};
	
	/** JSON serialization
	 *
	 */
	this.encodeJSON = function(){
        //Set instance variables
        var oSerialized = {
            operator: self.m_Data["operator"],
            checkTime: self.m_Data["checkTime"]
        };

        if(self.m_Data["source"] != undefined){
            oSerialized.source = self.m_Data["source"].getData().UId;
        }

        if(self.m_Data["destination"] != undefined){
            oSerialized.destination = self.m_Data["destination"].getData().UId;
        }
        
        return oSerialized;
    };
	
	
	/** Overwrite toString() method
     * @returns {string} String that describe Component object
     */
    this.toString = function(){
        return "[Relation] - Id: "+ self.id + ";";
    };

    /** Initial operations */
    init();
};