/**
 * @class Server
 *
 * @description Server communication object between page html and desktop application wrapper
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 *
 */
"use strict";

function Server(){
    //Build singleton instance
    if(this["singleton"]){
        console.warn("Singleton instance used");
        return this["singleton"];
    }
    this["singleton"] = this;

    
    /** Get current instance */
	var self = this;
	
	/**
	 * 
	 */
    this.queryExecute = function(strQuery){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }

        //--console.log("%cExecuting query: %c"+ strQuery,"color: blue;","color: orange;");
        window.eveDesk.queryExecute(strQuery);
    };
    
    /**
     * 
     */
    this.eve = {
        connect: function(ip,port,user,password,cbConnection){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }

            window["eve"]["ip"] = ip;

                //Add a function to a callback function
            //TODO In Debug
            cbConnection = (function(param) {
                var cached_function = cbConnection;
                window["cached_function"] = cbConnection;
                //console.log(cached_function);

                return function(param) {
                    // your code

                    //cached_function.apply(this, arguments); // use .apply() to call it
                    window["cached_function"].apply(this, arguments); // use .apply() to call it

                    // more of your code
                    switch(param){
                        case "Connection Ready":
                            $.publish("/view/application/refresh/connection",true);
                            break;
                        case "Connection State Try to connecting":
                        case "Connection State Control: Authentication success":
                        case "Connection State Authentication Failed":
                            break;
                        default:
                            //console.error(param);
                            //delete window["cached_function"];
                    }
                };
            }());
            
            return window.eveDesk.EVEConnect(ip,port,user,password,cbConnection);
        },
        disconnect: function(){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }
            var resp = window.eveDesk.EVEDisconnect();
            $.publish("/view/application/refresh/connection",false);

            window["eve"]["ip"] = undefined;

            return resp;
        },
        isConnect: function(){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }
            var resp = window.eveDesk.EVEIsConnected();

            return resp == "true" ? true : false;
        },
        getComponentsValue: function(){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }
            var resp = window.eveDesk.EVEQueryParametersUID();

            return resp == "true" ? true : false;
        },
        upload: function(sPath,cbUpload){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }

            return window.eveDesk.EVEUpload(sPath,cbUpload);
        },
        download: function(sPath,cbDownload){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }

            return window.eveDesk.EVEDownload(sPath,cbDownload);
        },
        copyFile: function(sPath,fileName,cbDownload){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }

            return window.eveDesk.EVECopyFile(sPath,fileName,cbDownload);
        },
        installPlugin: function(sPath,sInstallPath,cbUpload,cbInstall){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return;
            }

            return window.eveDesk.EVEInstallPlugin(sPath,sInstallPath,cbUpload,cbInstall);
        },
        md5: function(cbFunc){
            if(window.eveDesk == undefined){
                console.warn("This is not a desktop application release");
                return {
                    status: "error"
                };
            }
            return eveDesk.EVEDatabaseMD5(cbFunc);
        }
    };

    /** Load project.
     * If eveDesk, QT browser wrapper object don't exist, anything will be loaded
     *
     */
    this.loadProjectConfig = function(projectPath,dstDir){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        var path = (projectPath != undefined)?projectPath:"";
        var strResponse = JSON.parse( window.eveDesk.loadProjectConfig(path,dstDir) );

        /* if(strResponse.status.toLowerCase() != "ok"){
        if(strResponse.status != "ok"){
            return;
        }*/

        return strResponse;
    };
    
    /** Load project.
     * If eveDesk, QT browser wrapper object don't exist, anything will be loaded
     * 
     */
    this.loadProject = function(projectPath,dstDir){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        //--console.log("Executing %cLoad Project","color: blue;");
        var path = (projectPath != undefined)?projectPath:"";
        var strResponse = window.eveDesk.loadProject(path,dstDir);
        
        return JSON.parse(strResponse);
    };
    
    /** Save project.
     * If eveDesk, QT browser wrapper object don't exist, anything will be saved
     * 
     */
    this.saveProject = function(json,projectPath,dstDir){

        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        //DEBUG
        //self.writeFile("./DEBUG.json",json);
        //self.writeFile("./project.data",json);
        self.writeFile("../project/project.data",json);
        //DEBUG

        var path = (projectPath != undefined)?projectPath:"";

        //--console.log("Executing %cSave Project","color: blue;");
        
        var strResponse = window.eveDesk.saveProject(json,path,dstDir);
        //console.warn(strResponse);
        
        var oResponse = JSON.parse(strResponse);

        //console.warn(oResponse);
        //Overwrite project.data file
        if(oResponse.status == "ok"){
            //self.writeFile("../project/project.data",json);
        }
        
        return oResponse;
    };    
    
    /** Clear project directory
     * 
     */
    this.clearProject = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
        }

        //--console.log("Executing %cClear Project","color: blue;");
        
        eveDesk.clearProject();
    };
    
    /** Copy file.
     * @param {string} strSource Source path of file
     * @param {string} strDest Destination path of file
     */
    this.copyFile = function(strSource,strDest){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        //Copy file
        var bCheck = window.eveDesk.fileCopy(strSource,strDest);

        //console.error(bCheck);
        //TODO farsi ritornare da Dario un oggetto con lo stato
        if(!bCheck){
            console.warn("Can't copy \"%c"+ strSource +"%c\" file.\n File already exist or path don't exist","color:black;","color:inherit;");
        }
        
        return bCheck;
    };
    
    
    this.selectImage = function(width,height,optional){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        var path = window.eveDesk.selectImage(width,height);
        //console.log(path);
        var res = path.split(",");

        //console.log(res[0]);
        //console.log(res[1]);
        //console.log(res[2]);
        if (optional!=undefined)
        {
            optional.width=res[1];
            optional.height=res[2];

        }



        return res[0];
    };
    this.selectPath = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        window.eveDesk.choosePath();
    };
    
    this.readAnyFile = function(ext){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }

        return window.eveDesk.readAnyFile(ext);
    };
    this.readFile = function(path){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        return window.eveDesk.readFile(path);
    };

    
    this.getOpenFileName = function (title,path,filter) {
        if (window.eveDesk == undefined) {
            console.warn("This is not a desktop application release");
            return; 
        }

        return window.eveDesk.getOpenFileName(title,path,filter);
    };

    this.readAbsoluteFile = function (path) {
        if (window.eveDesk == undefined) {
            console.warn("This is not a desktop application release");
            return;
        }

        return window.eveDesk.readAbsoluteFile(path);
    };
    
    this.getSaveFileName = function (title,path,filter) {
        if (window.eveDesk == undefined) {
            console.warn("This is not a desktop application release");
            return;
        }

        return window.eveDesk.getSaveFileName(title,path,filter);
    };

    this.writeFile = function(path,content){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        return window.eveDesk.writeFile(path,content);
    };

    this.writeAbsoluteFile = function (path,data) {
        if (window.eveDesk == undefined) {
            console.warn("This is not a desktop application release");
            return;
        }

        return window.eveDesk.writeFileAbsolute(path,data);
    };

    //Delete file (relative path)
    this.deleteFile = function(path){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        return window.eveDesk.deleteFile(path);
    };
    
    this.exit = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return;
        }
        
        window.eveDesk.exit();
    };

    /**
	 * 
	 */
    this.queryResponse = function(json){
        //--console.log("%cServer response", "color: blue;");
        //--console.log(json);
    };
    this.projectFolder = function(folder){
        //--console.log("%cProject path:", "color: blue;");
        //--console.log(folder);
    };
    
    this.fileList = function(sRelPath){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        
        return JSON.parse( eveDesk.fileList(sRelPath) );
    };
    
    this.applicationPath = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        
        return eveDesk.applicationPath();
    };

    this.selectFolder = function(strTitle){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        return eveDesk.selectFolder(strTitle);
    };

    /** Save eve manager group file (EMG)
     *
     * @param strTitle
     * @param strDir
     * @param strExt
     * @param strContent
     * @returns {*}
     */
    this.saveEMGFile = function(strDir,strContent){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        return eveDesk.saveFile("Save Group",strDir,"EVEManager group (*.emg)",strContent);
    };

    this.updateParameter = function(string){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        //console.log(string);
        return eveDesk.updateParameter(string);
    };

    this.sendParameterValue = function(sUId,sType,sValue){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        //console.log(string);
        return eveDesk.sendParameterValue(sUId,sType,sValue);
    };

    this.setTitle = function(str){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        eveDesk.setWindowTitle(str);
    };

    this.MD5 = function(prjFile){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        return eveDesk.MD5(prjFile);
    };

    this.getQTRelease = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        return eveDesk.getQTRelease();
    };

    this.downloadFile = function(srcPath,dstPath,cbFunc){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }
        return eveDesk.downloadFile(srcPath,"0",dstPath,cbFunc);
    };

    this.abortLoading = function(){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        //TODO enable abortLoading
        return eveDesk.abortLoading();
    };

    this.discover = function(cbFunc){
        if(window.eveDesk == undefined){
            console.warn("This is not a desktop application release");
            return {
                status: "error"
            };
        }

        //TODO enable abortLoading
        return eveDesk.discover(cbFunc);
    };
/*
    window.onload = function() {
            var baseUrl = (/[?&]webChannelBaseUrl=([A-Za-z0-9\-:/\.]+)/.exec(location.search)[1]);
        console.log("Connecting to WebSocket server at " + baseUrl + ".");
            var socket = new WebSocket(baseUrl);

            socket.onclose = function()
            {
                console.error("web channel closed");
            };
            socket.onerror = function(error)
            {
                console.error("web channel error: " + error);
            };
            socket.onopen = function()
            {
                console.log("WebSocket connected, setting up QWebChannel.");
                new QWebChannel(socket, function(channel) {
                    // make dialog object accessible globally
                    window.dialog = channel.objects.dialog;

                    document.getElementById("send").onclick = function() {
                        var input = document.getElementById("input");
                        var text = input.value;
                        if (!text) {
                            return;
                        }

                        console.log("Sent message: " + text);
                        input.value = "";
                        dialog.receiveText(text);
                    }

                    dialog.sendText.connect(function(message) {
                        console.log("Received message: " + message);
                    });

                    dialog.receiveText("Client connected, ready to send/receive messages!");
                    console.log("Connected to WebChannel, ready to send/receive messages!");
                });
            }
        }*/
            //END SETUP

}