/** Javascript Vivaldi Group Importer
 *
 * 5/3/2014 by Andreosky Padovanosky
 *
 */
"use strict";

var VERSION = "v.1.0.6";
var NAME = "Vivaldi Group Wizard " + VERSION;
var $ = jQuery;

try {
    console.log("%cStart " + NAME, "color:DarkSlateBlue");
    create_gui();
    console.log("%cEnd " + NAME, "color:DarkSlateBlue");
} catch (err) {
    console.error(NAME + " error\n" + err);
    alert(NAME + " error\n" + err);
}

function create_gui(){
    var wizard = $("<div id='vivaldigroup-wizard' title='" + NAME + "'></div>");
    wizard.append("</br><label>Name</label></br>");
    wizard.append("<input type='text' id='gtw-name' value='Vivaldi Giove'/></br>");
    wizard.append("</br><label>Address</label></br>");
    wizard.append("<input type='text' id='gtw-address' value='192.168.0.101'/></br>");
    wizard.append("</br><label>Port</label></br>");
    wizard.append("<input type='text' id='gtw-port' value='10001' onkeypress='bigh.onlyPosInt();' /></br>");
    wizard.append("</br><label>Giove ID</label></br>");
    wizard.append("<input type='text' id='giove-id' value='1' onkeypress='bigh.onlyPosInt();' /></br>");
    wizard.append("</br><label>Numbers of controllers</label></br>");
    wizard.append("<input type='text' id='controllers-count' value='10' onkeypress='bigh.onlyPosInt();' /></br>");
    $("body").append(wizard);

    wizard.dialog({
        modal:true,
        close: function() {
            $( this ).remove();
        },
        buttons: {
        "Create Componens...": function() {
            $( this ).dialog( "close" );
            $( this ).remove();

            blr.executionStack.show(NAME);
            blr.executionStack.push(
                function (in_args,out_args) {
                    out_args = create_system(
                        in_args.gateway,
                        in_args.address,
                        in_args.port,
                        in_args.giove,
                        in_args.count
                    );
                },
                {
                    gateway:$(this).children("#gtw-name").val(),
                    address:$(this).children("#gtw-address").val(),
                    port:$(this).children("#gtw-port").val(),
                    giove:$(this).children("#giove-id").val(),
                    count:$(this).children("#controllers-count").val()
                }
            );
            blr.executionStack.execute(1,false,false);
        },
        Cancel: function() {
            $( this ).dialog( "close" );
            $( this ).remove();
        }
      }
    });

    return wizard;
}

function create_system(name, ip, port, gioveID, numControllers) {
    if (numControllers > 60) {
        numControllers = 60;
    }

    var oGateway = getGateway(name, ip, port, gioveID);
    var oGroup = getMainGroup(name);

    blr.executionStack.push(
        function (in_args,out_args) {
            out_args = createGiove(
                in_args.gateway,
                in_args.group
            );
        },
        {
            gateway:oGateway,
            group:oGroup
        }
    );

    for (var i = 0; i < numControllers; ++i) {
        blr.executionStack.push(
            function (in_args,out_args) {
                out_args = createRoomController(in_args.gateway,in_args.group,in_args.index);
            },
            {
                gateway:oGateway,
                group:oGroup,
                index:i + 1
            }
        );
    }

    blr.executionStack.execute(100,true);
}

function getGateway(name, address, port, gioveid) {
    var oGateway = undefined;
    // Create Gateway
    var aGateways = application.getGatewaysByProtocol("vivaldi");
    if (aGateways.length > 0) {
        oGateway = aGateways[0];
    }

    if (oGateway == undefined) {
        oGateway = application.addGateway({
            name: name,
            protocol: "vivaldi"
        });
    }
    oGateway.data.setName(name);
    oGateway.data.setParamVal("vivaldi/address", address);
    oGateway.data.setParamVal("vivaldi/port", port);
    oGateway.data.setParamVal("vivaldi/device-gid", gioveid);
    return oGateway;
}

function getMainGroup(name) {
    var oGroup = application.addComponent({
        type: "group",
        name: name
    });
    return oGroup;
}

function createGiove(gateway,group) {
    var oGroup = application.addComponent({
        type: "group",
        name: "CA20/21"
    }, group);

    ///////////////////////////////////////////
    // Scenari
    ///////////////////////////////////////////
    var oGroupScenaries = application.addComponent({
        type: "group",
        name: _D("Scenaries")
    }, oGroup);
    for (var i = 0; i < 16; ++i) {
        var id = i + 1;
        var oItem = application.addComponent({
            type: "scenario",
            gateway: gateway,
            name: _D("Scene " + id),
            "vivaldi/device-fid": 3,
            "vivaldi/device-id": 1,
            "vivaldi/device-pid": 1,
            "vivaldi/value": id
        }, oGroupScenaries);
    }
    ///////////////////////////////////////////
    // IR
    ///////////////////////////////////////////
    var oGroupIR = application.addComponent({
        type: "group",
        name: _D("IR")
    }, oGroup);
    for (var i = 0; i < 50; ++i) {
        var id = i + 1;
        var oItem = application.addComponent({
            type: "scenario",
            gateway: gateway,
            name: _D("IR " + id),
            "vivaldi/device-fid": 5,
            "vivaldi/device-id": 1,
            "vivaldi/device-pid": 1,
            "vivaldi/value": id,
            "icons": [blr.gfxResource.getIconByTypeName(1, "circle", 200, 0, 0)]
        }, oGroupIR);
    }
    return oGroup;
}

function createRoomController(gateway, group, id) {
    var oItem = null;
    var oGroup = application.addComponent({
        type: "group",
        name: _D("RC") + " " + id 
    }, group);

    // Power
    oItem = application.addComponent({
        type: "switch",
        gateway: gateway,
        name: id + " " + _D("Power"),
        "vivaldi/device-fid": 0,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "off"),
            blr.gfxResource.getIconByTypeName(1, "on")
        ]
    }, oGroup);
    // Mute
    oItem = application.addComponent({
        type: "switch",
        gateway: gateway,
        name: id + " " + _D("Mute"),
        "vivaldi/device-fid": 10,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "mute"),
            blr.gfxResource.getIconByTypeName(1, "sound")
        ]
    }, oGroup);
    // Sources
    oItem = application.addComponent({
        type: "combo",
        gateway: gateway,
        name: id + " " + _D("Source"),
        "vivaldi/device-fid": 4,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "values": [{
            "value": 1,
            "description": _D("Tuner")
        }, {
            "value": 2,
            "description": _D("Cd")
        }, {
            "value": 3,
            "description": _D("Dvd")
        }, {
            "value": 4,
            "description": _D("Tv")
        }, {
            "value": 5,
            "description": _D("Dvr")
        }, {
            "value": 6,
            "description": _D("Aux 1")
        }, {
            "value": 7,
            "description": _D("Aux 2")
        }, {
            "value": 8,
            "description": _D("Mp3")
        }, {
            "value": 9,
            "description": _D("Aux In")
        }, {
            "value": 10,
            "description": _D("Mic Base")
        }, {
            "value": 11,
            "description": _D("Usb")
        }, {
            "value": 12,
            "description": _D("Mic In")
        }, {
            "value": 13,
            "description": _D("Mic Out")
        }, {
            "value": 14,
            "description": _D("Bluetooth")
        }],
        "icons": [
            blr.gfxResource.getIconByTypeName(1, "tuner"),
            blr.gfxResource.getIconByTypeName(2, "cd"),
            blr.gfxResource.getIconByTypeName(3, "dvd"),
            blr.gfxResource.getIconByTypeName(4, "tv"),
            blr.gfxResource.getIconByTypeName(5, "dvr"),
            blr.gfxResource.getIconByTypeName(6, "one"),
            blr.gfxResource.getIconByTypeName(7, "two"),
            blr.gfxResource.getIconByTypeName(8, "mp3"),
            blr.gfxResource.getIconByTypeName(9, "in"),
            blr.gfxResource.getIconByTypeName(10, "mic"),
            blr.gfxResource.getIconByTypeName(11, "usb"),
            blr.gfxResource.getIconByTypeName(12, "mic_in"),
            blr.gfxResource.getIconByTypeName(13, "mic_out"),
            blr.gfxResource.getIconByTypeName(14, "bluetooth")
        ]
    }, oGroup);
    // Volume
    oItem = application.addComponent({
        type: "slider",
        gateway: gateway,
        name: id + " " + _D("Volume"),
        "vivaldi/device-fid": 1,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [blr.gfxResource.getIconByTypeName(0, "volume")]
    }, oGroup);

    // Bass
    oItem = application.addComponent({
        type: "slider",
        gateway: gateway,
        name: id + " " + _D("Bass"),
        "vivaldi/device-fid": 3,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [blr.gfxResource.getIconByTypeName(0, "bass")]
    }, oGroup);

    // Treble
    oItem = application.addComponent({
        type: "slider",
        gateway: gateway,
        name: id + " " + _D("Treble"),
        "vivaldi/device-fid": 2,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [blr.gfxResource.getIconByTypeName(0, "treble")]
    }, oGroup);

    // Tuner
    oItem = application.addComponent({
        type: "tuner",
        gateway: gateway,
        name: id + " " + _D("Tuner"),
        digits: 1,
        min: "87.5",
        max: "107.9",
        nolabel: false,
        step: "0.1",
        suffix: "MHz",
        "vivaldi/device-fid": 9,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "icons": [blr.gfxResource.getIconByTypeName(0, "tuner")]
    }, oGroup);

    var oGatewayGUI = application.getGatewaysByProtocol("gui")[0];

    ///////////////////////////////////////////
    // Track
    ///////////////////////////////////////////
    var aComponents = [0, 0];
    var oGroupTrack = application.addComponent({
        type: "group",
        name: id + " " + _D("Track")
    }, oGroup);
    // Track Prev
    oItem = application.addComponent({
        type: "scenario",
        gateway: gateway,
        name: id + " " + _D("Prev"),
        "vivaldi/device-fid": 8,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "vivaldi/value": 101
    }, oGroupTrack);
    aComponents[0] = oItem.data.UId;
    // Track Next
    oItem = application.addComponent({
        type: "scenario",
        gateway: gateway,
        name: id + " " + _D("Next"),
        "vivaldi/device-fid": 8,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "vivaldi/value": 102
    }, oGroupTrack);
    aComponents[1] = oItem.data.UId;
    // Seeker Tracks
    oItem = application.addComponent({
        type: "seeker",
        gateway: oGatewayGUI,
        name: id + " " + _D("Track"),
        "widget/components": aComponents,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "track_prev"),
            blr.gfxResource.getIconByTypeName(1, "track_next")
        ]
    }, oGroup);

    ///////////////////////////////////////////
    // CD
    ///////////////////////////////////////////
    aComponents = [0, 0];
    var oGroupCD = application.addComponent({
        type: "group",
        name: id + " " + _D("CD")
    }, oGroup);
    // CD Prev
    oItem = application.addComponent({
        type: "scenario",
        gateway: gateway,
        name: id + " " + _D("Prev"),
        "vivaldi/device-fid": 7,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "vivaldi/value": 101
    }, oGroupCD);
    aComponents[0] = oItem.data.UId;
    // CD Next
    oItem = application.addComponent({
        type: "scenario",
        gateway: gateway,
        name: id + " " + _D("Next"),
        "vivaldi/device-fid": 7,
        "vivaldi/device-id": id,
        "vivaldi/device-pid": 2,
        "vivaldi/value": 102
    }, oGroupCD);
    aComponents[1] = oItem.data.UId;
    // CD Tracks
    oItem = application.addComponent({
        type: "seeker",
        gateway: oGatewayGUI,
        name: id + " " + _D("CD"),
        "widget/components": aComponents,
        "icons": [
            blr.gfxResource.getIconByTypeName(0, "cd_prev"),
            blr.gfxResource.getIconByTypeName(1, "cd_next")
        ]
    }, oGroup);

    return oGroup;
}


